.include "2313def.inc"

;    0=0x0a, *=0x0b, #=0x0c
.equ CH_STAR = 0x0b
.equ CH_DIEZ = 0x0c

.equ MAXBUFF = 10
.equ NPWDDIGITS = 8;   

;  3    ,   
.equ Timer1HighByte=0
.equ Timer1LowByte=0

.dseg
inputseq:	.byte 	MAXBUFF ; data to be received from dialer
tmr1_ovf_count: .byte 1

.def dialdigit=r9
.def sreg_int=r10 ; place to save status register
.def RelayState=r15;  
.def temp =r16
.def arg1 =r17;argument for functions call
.def temp_int=r18
.def countReceived=r19;   
.def mode=r20;   : 0 -  , 
;/1 -   ,/ 1 -   , 
;2 -  
.def ReadyToDial=r21;     
.def _delay0=r22
.def _delay1=r23
.def _delay2=r24
.def arg2=r25
;26...31 = X...Z

.equ led_port = PORTB
.cseg

.macro Relay_on
	sbi PORTD, 0;    
	ser temp
	mov RelayState, temp
	sbi PORTB, 7 ;  
.endm	

.macro Relay_off
	cbi PORTD, 0
	clr temp
	mov RelayState, temp
	cbi PORTB, 7 ;  
.endm	

.org 0
reset:
	rjmp start ; Reset Handler
	rjmp int0_proc ;EXT_INT0 ; IRQ0 Handler
	reti ;EXT_INT1 ; IRQ1 Handler
	reti  ;TIM_CAPT1 ; Timer1 Capture Handler
	reti  ;TIM_COMP1 ; Timer1 Compare Handler
	rjmp tmr1_ovf  ;TIM_OVF1 ; Timer1 Overflow Handler
	reti  ;TIM_OVF0 ; Timer0 Overflow Handler
	reti  ;UART_RXC ; UART RX Complete Handler
	reti  ;UART_DRE ; UDR Empty Handler
	reti  ;UART_TXC ; UART TX Complete Handler
	reti  ;ANA_COMP ; Analog Comparator Handler
;
start:
	cli
	; interrupts disabled
	ldi temp,low(RAMEND)
	out SPL,temp
;	ldi temp,high(RAMEND) ;does not exist in 2313
;	out SPH,temp	;init Stack Pointer

	ser temp
	out DDRB,temp	;Set direction out
	ldi temp, 0b00000011; D0, D1  
	out DDRD, temp
	
	Relay_off; 
	
	;   
	clr mode	
	clr ReadyToDial
	ldi countReceived, MAXBUFF
	rcall ResetInputData;    

	ldi arg1, 38;' '; 'H' hangup
	rcall out_digit

	clr temp
	sts tmr1_ovf_count, temp
	out GIMSK, temp  	;disable external interrupt
	
	ldi temp, (1<<ISC01) | (1<<ISC00) ;  INT0 enable
	out MCUCR, temp
	
	ldi temp, (1<<INT0)
	out GIMSK, temp  	;enable external interrupt
	
	ldi temp, (1<<TOIE1) | (1<<TOIE0); enable timer interrupt
	out TIMSK, temp

	ldi temp, (1<<CS02) | (1<<CS00); set prescaler to CK/1024
	out TCCR0, temp
	
	ldi temp, 5 ; set initial 8-bit timer 0
	out TCNT0, temp
	
	clr temp ;  1
	out TCCR1A, temp
	ldi temp, (1<<CS12)|(1<<CS10) ;  - /1024
	out TCCR1B, temp

	rcall ResetTimer1	

;     ,      
;	ldi _delay0, 10
;	rcall _delay0x50msec_delay;   - 250ms
	rcall one_sec_delay
	in temp, PIND
	andi temp, 0b01111000
	cpi temp, 0b01011000;  //  ?
	brne skip_greeting
	
	ldi arg1, low(msg_hello*2)
	ldi arg2, high(msg_hello*2)
	rcall print_msg

	ldi arg1, low(msg_pass_eq*2)
	ldi arg2, high(msg_pass_eq*2)
	rcall print_msg
	rcall show_password

skip_greeting:
	ldi arg1, 38;' '; 'H' hangup
	rcall out_digit
	
	ser temp
	out GIFR, temp; clear interrupts if any
	sei	;global interrupt enable
	
loop: ;  
	cpi ReadyToDial, 0;     
	breq loop
	
	Relay_on ;   
	ldi arg1, 13; 'd' dialing
	rcall out_digit

	rcall one_sec_delay;  1 

;  
	ldi XL, low(inputseq)
	ldi XH, high(inputseq)
	ldi countReceived, 7
	
dialloop0:
	ld dialdigit, X+

	mov arg1, dialdigit; ,  
	rcall out_digit

dialloop1:
	Relay_off
	ldi _delay0, 1
	rcall _delay0x50msec_delay ;50  
	Relay_on
	ldi _delay0, 1
	rcall _delay0x50msec_delay;50  
	
	cpi ReadyToDial, 0
	breq breakdialloop

	dec dialdigit
	brne dialloop1

	rcall one_sec_delay;   

;	cli ;  ,   .

;     0
;	cpi countReceived, MAXBUFF
;	brge enddial
;	sbrc countReceived, 7;   
;	rjmp enddial
;	cpi countReceived, 0;     ?
;	breq enddial

;	sei ;  
	dec countReceived
	brne dialloop0
enddial:	
	clr ReadyToDial
	rcall ResetInputData
;	sei ; 
	rjmp loop

breakdialloop:	;    
	Relay_off
	rcall ResetInputData
	rjmp loop

;      
; :  arg1 -     digits.
; : .
out_digit:
	push ZH
	push ZL
	ldi	ZH,high(2*digits)	; Load high part of byte address into ZH
	ldi	ZL,low(2*digits)	; Load low part of byte address into ZL
	
	add	ZL, arg1
	brcc PC+2;  carry,     256    ZH
	inc ZH
	
	lpm	
	out 	led_port, r0
	pop ZL
	pop ZH
	ret

; 1- 
one_sec_delay:
	ldi	_delay0, 20 ;20 = 1s @ 4 MHz
;  ,   _delay0	
;  ..  1   !!!
_delay0x50msec_delay:
	ldi	_delay1, 255
	ldi	_delay2, 255
delay:	dec	_delay2
	brne	delay
	dec	_delay1
	brne	delay
	dec	_delay0
	brne	delay
	ret
short_delay:
	ldi	_delay2, 50
s_delay:	dec	_delay2
	brne	s_delay
	ret
_1ms_delay:;   1 
	ldi	_delay0, 1
	ldi	_delay1, 6
	ldi	_delay2, 160
	rjmp delay
;        
; -  
ResetInputData:
;	ldi countReceived, MAXBUFF; .   ,  
;	push temp
;	ldi XL, low(inputseq)
;	ldi XH, high(inputseq)
;	clr temp
	;countReceived   ,   .
loopRID: ; 
;	st X+, temp
;	dec countReceived
;	breq endlRID;   , 
;	rjmp loopRID
endlRID:	
	ldi XL, low(inputseq)
	ldi XH, high(inputseq)
	clr countReceived
	clr ReadyToDial
;	pop temp
	ret



;   DTMF .   DTMF 
;18    DSO,      .
int0_proc:
	in sreg_int, SREG; save SREG
	rcall short_delay;     
	in temp_int, PIND ; get d6...d3
	lsr temp_int
	lsr temp_int
	lsr temp_int;now have code in d3...d0
	mov arg1, temp_int
	cpi arg1, 0x0a;    ?
	breq zero_pushed;   
	rjmp cont_1
zero_pushed:	
	clr arg1;    0
cont_1:	
	rcall out_digit
	cbi PORTB, 7 ;  
tst_sig_on:
	sbic PIND, 2; ,      
	rjmp tst_sig_on	
	sbi PORTB, 7 ;  

	ldi arg1, 38;    
	rcall out_digit

	ldi _delay0, 3
	rcall _delay0x50msec_delay ;150  
	rcall beep_ack; ,   
	ldi _delay0, 4 ;  200 ,    18 
	rcall _delay0x50msec_delay ;150  
	
	rcall ResetTimer1 ;  1,   
	
	;     
	cpi mode, 0 ;  
	breq i0_get_pass
	cpi mode, 1
	breq i0_get_num
	cpi mode, 2
	breq i0_process_call
	rjmp i0_ret
i0_get_pass:
;	push ZH
;	push ZL
;	pop ZL
;	pop ZH
	cpi temp_int, CH_STAR;  ?
	breq i0_ret;    -    

	cpi temp_int, CH_DIEZ;  ?
	breq i0_clear_pwd;    

	st X+, temp_int;     .  
	inc countReceived;   
	cpi countReceived, NPWDDIGITS;    ?
	brne i0_ret

	rcall CheckPwd;    EEPROM 
	brne i0_clear_pwd_1 ;   ,   

;	ldi arg1, 22; 'P' pwd passed
;	rcall out_digit
	
	ldi mode, 1;   -   
	rcall ResetInputData
	rjmp i0_ret
i0_get_num:
	cpi temp_int, CH_STAR;  ?
	breq i0_ret;    -    

	cpi temp_int, CH_DIEZ;  ?
	breq i0_clear_input;   

	st X+, temp_int;     .  
	inc countReceived;   
	cpi countReceived, 1;  ?
	breq check_local;,     8   0
	rjmp localcall;    1 
check_local:	
	cpi temp_int, 8;  -  ?
	breq i0_clear_input;-
	cpi temp_int, 0x0a;  -  ?
	breq i0_clear_input;-
	rjmp localcall;  
localcall:	
;      .    
;,     .
	cpi countReceived, 7 ;7   
	brge dial_number; 
	rjmp i0_ret;    
dial_number:	
;      
	rcall beep_ok
	ser ReadyToDial
	ldi countReceived, 7;  ,     
	ldi mode, 2
	rjmp i0_ret
i0_process_call:
	cpi temp_int, CH_DIEZ;  ? hangup.
	brne i0_ret
;	rjmp reset;   
	ldi mode, 1
	clr ReadyToDial;   
;	clr countReceived  
	Relay_off
	rjmp i0_clear_input

i0_clear_pwd_1:	
;	ldi arg1, 11; 'b' bad pwd
;	rcall out_digit
;	rcall one_sec_delay
i0_clear_pwd:	
;	ldi arg1, low(msg_cleared*2)
;	ldi arg2, high(msg_cleared*2)
;	rcall print_msg
	
;	rcall beep_error	
	rcall ResetInputData;   
	clr mode;    0 -  
	rjmp i0_ret
i0_clear_input:	
;	ldi arg1, low(msg_cleared*2)
;	ldi arg2, high(msg_cleared*2)
;	rcall print_msg
	
	rcall beep_error	
	rcall ResetInputData;   
	ldi mode, 1;    1 -  
	rjmp i0_ret
i0_ret:		

	ser temp_int
	out GIFR, temp_int; clear interrupts if any
	
	out SREG, sreg_int; restore SREG
	reti
;        EEPROM
; Z=1,   .
CheckPwd:
	sbic	EECR,EEWE	;if EEWE not clear
	rjmp	CheckPwd	;    wait more

;	ldi 	temp_int, password
	ldi 	temp_int, low(password);    
	out	EEAR, temp_int	;output address 

	ldi XL, low(inputseq)
	ldi XH, high(inputseq)

	ldi temp_int, NPWDDIGITS
loopCkPwd:	
	sbi	EECR,EERE	;set EEPROM Read strobe
				;This instruction takes 4 clock cycles since
				;it halts the CPU for two clock cycles
	in	r1, EEDR	;   
	ld	r2, X+		;  
	cpse r1, r2;  ,
	rjmp badPwd
	rjmp contloopCkPwd;
badPwd:
;	ldi arg1, low(msg_bad_pass*2)
;	ldi arg2, high(msg_bad_pass*2)
;	rcall print_msg
	rcall beep_error
	
	clz ;  z -   !
	ret
contloopCkPwd:	

	in r0, EEAR;  
	inc r0
	out EEAR, r0

	dec temp_int;  
	breq pwdOk

	rjmp loopCkPwd
pwdOk:
;	ldi arg1, low(msg_passed*2)
;	ldi arg2, high(msg_passed*2)
;	rcall print_msg
	rcall beep_ok
	
	sez
	ret
;   1    3 
ResetTimer1:
	push temp
	ldi temp, Timer1HighByte; set counter high byte - order is significant!
	out TCNT1H, temp
	ldi temp, Timer1LowByte; set counter low byte - order is significant!
	out TCNT1L, temp
	clr temp
	sts tmr1_ovf_count, temp
	pop temp
	ret
	
;  16-  1.
tmr1_ovf:
	in sreg_int, SREG; save SREG
	push temp_int
	lds temp_int, tmr1_ovf_count
	inc temp_int

	cpi temp_int, 9;      
	;  ,    
	breq tmr1_ovf_beep

	cpi temp_int, 10;  10  ,  180 
	brne tmr1_ovf_ok2
	
	rjmp reset;   
tmr1_ovf_beep:
	rcall beep_error
	;  	
tmr1_ovf_ok2:
	sts tmr1_ovf_count, temp_int
	pop temp_int
;	Relay_off
;	rcall ResetInputData
;	rcall ResetTimer1
	out SREG, sreg_int; restore SREG
	reti

;  ASCII      
; - arg1 - ascii 
; - arg1 -   . 
;translate_char:
;	ldi YL, low(chars)
;	ldi YH, high(chars)
;	clr r0
;	ret
	
;   
;: arg1, arg2 -  	(low, high)
; , low(bad_password*2), high(bad_password*2)
print_msg:
	push ZL
	push ZH
	mov ZL, arg1
	mov ZH, arg2
loop_print_msg:
	lpm ;   r0
	adiw r30, 1; incw Z
	mov arg1, r0
	cpi arg1, 0
	breq exit_print_msg;   -  - 
	rcall out_digit 

	ldi _delay0, 4;  0.2   
	rcall _delay0x50msec_delay

	ldi arg1, 38;    
	rcall out_digit 

	ldi _delay0, 1;  0.05  - 
	rcall _delay0x50msec_delay

	rjmp Loop_print_msg
exit_print_msg:
	pop ZH
	pop ZL
	ret

;    
show_password:
	push temp
	sbic	EECR,EEWE	;if EEWE not clear
	rjmp	show_password;    wait more

	ldi 	temp, low(password);    
	out	EEAR, temp	;output address 

	ldi temp, NPWDDIGITS
loopshpw:	
	sbi	EECR,EERE	;set EEPROM Read strobe
	in	arg1, EEDR	;   
	rcall out_digit

	ldi _delay0, 4;  0.2   
	rcall _delay0x50msec_delay

	ldi arg1, 38;    
	rcall out_digit 

	ldi _delay0, 1;  0.05  - 
	rcall _delay0x50msec_delay

	in r0, EEAR;  
	inc r0
	out EEAR, r0

       	dec temp
	brne loopshpw
	
	pop temp
	ret

;      B0
; : arg1 -    
beep:
	push temp
	clr temp
	sbic PORTB, 0;    0    r9
	ser temp
	
l_b_ack_0:
	sbi PORTB, 0
	rcall _1ms_delay
	cbi PORTB, 0
	rcall _1ms_delay
	
	dec arg1
	brne l_b_ack_0

	sbrc temp, 0;    0  
	sbi PORTB, 0
	
	pop temp
	ret

;      200 ,  
beep_ok:
	ldi arg1, 250
	rcall beep
	ret

beep_ack:
	ldi arg1, 15
	rcall beep
	ret
	
;       200 ,   ()
beep_error:
	push temp
	ldi arg1, 200
	rcall beep
	ldi _delay0, 4
	rcall _delay0x50msec_delay

	ldi arg1, 200
	rcall beep
	ldi _delay0, 4
	rcall _delay0x50msec_delay

	ldi arg1, 200
	rcall beep
	ldi _delay0, 4
	rcall _delay0x50msec_delay

	pop temp
	ret
;    	
;chars:
;	.db '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 
;	.db 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'J', 'L', 
;	.db 'N', 'O', 'P', 'R', 'S', 'T', 'U', 'Y', '', '', '', 
;	.db '', '', '-', '_', '=', '@', '\x00'
digits:	
	.db 0b11000000, 0b11111001, 0b10100100, 0b10110000 ;0:  0123
	.db 0b10011001, 0b10010010, 0b10000010, 0b11111000 ;4:  4567
	.db 0b10000000, 0b10010000, 0b10001000, 0b10000011 ;8:  89Ab
	.db 0b11000110, 0b10100001, 0b10000110, 0b10001110 ;12: dEF
	.db 0b11000010, 0b10001011, 0b11110001, 0b11000111 ;16: GhJL
	.db 0b10101011, 0b10100011, 0b10001100, 0b10101111 ;20: nor
	.db 0b10000111, 0b11000001, 0b10010001, 0b11001110 ;24: tUY
	.db 0b10101001, 0b11001000, 0b10100111, 0b10000011 ;28: 
	.db 0b10111111, 0b11110111, 0b11110110, 0b10100000 ;32: -_=@
	.db 0b01111111, 0b10001001, 0b11111111, 0b11000000; 36: , H <space> O
;     0!
msg_bad_pass:
	.db 11, 10, 13, 33, 22, 10, 5, 5, 38, 0
;	    b   A   d   _   P   A   S=5 S  	
msg_passed:
	.db 22, 10, 5, 5, 14, 13, 0
msg_cleared:
	.db 12, 19, 14, 10, 23, 14, 13, 0	
msg_hello:
	.db 37, 14, 19, 19, 39, 0	
msg_pass_eq:; "pass="
	.db 22, 10, 5, 5, 34, 0

.eseg
;  ..  NPWDDIGITS
.org 0
password: 
	.db 0x03,0x08,0x06,0x03,0x04,0x02,0x08,0x03; "dtmfgate"    
eePulse:  
	.db 1 ; init eePulse as 1
